import java.io.*;
import java.text.*;
import java.util.*;
import java.lang.reflect.*;
import gsfc.nssdc.cdf.*;
import gsfc.nssdc.cdf.util.*;

/**
 *  This program demonstrates how to read the contents of test.cdf created 
 *  by CreateCDF.java in this directory.
 */

public class ReadCDF implements CDFConstants{

    public static void main(String[] args) {
        String  fileName = "test";
        int     maxVarNameLength = 22;

        try {
            CDF cdf = null; 
            if (args.length == 0) {
                cdf  = CDF.open(fileName, READONLYoff);
            }
            else {
                fileName = args[0];
                cdf = CDF.open(fileName, READONLYoff);
            }

            /************************************************************/
            /* If a decoding method is not specified when a CDF file is */
            /* opened, the CDF libaray knows what encoding method was   */
            /* used to create the CDF file.                             */
            /*                                                          */
            /* Decoding method should be specified only if one needs    */
            /* to translate data from one platform to another.          */ 
            /************************************************************/
            // cdf.selectDecoding(NETWORK_DECODING);

            /**********************************/
            /* Print out the file information */
            /**********************************/
            System.out.println("File Info\n"+
                               "=========================================");

            if (cdf.confirmReadOnlyMode() == READONLYon)
                System.out.println("CDF File:     "+cdf+" (READONLYon)");
            else {
                System.out.println("CDF File:     "+cdf+" (READONLYoff)");
            }

            System.out.println("Version:      "+cdf.getVersion());
            String cp = cdf.getCopyright();
            System.out.println("Copyright:    "+cp);
            System.out.println("Format:       "+CDFUtils.getStringFormat(cdf));
            System.out.println("Encoding:     "+
                               CDFUtils.getStringEncoding(cdf));
            System.out.println("Decoding:     "+
                               CDFUtils.getStringDecoding(cdf));
            System.out.println("Majority:     "+
                               CDFUtils.getStringMajority(cdf));
            System.out.println("numRvars:     "+cdf.getNumRvars());
            System.out.println("numZvars:     "+cdf.getNumZvars());
            System.out.println("numAttrs:     "+cdf.getNumAttrs()+
                               " ("+cdf.getNumGattrs()+" global, "+
                               cdf.getNumVattrs()+" variable)");
            System.out.println("Compression:  "+cdf.getCompression());
            System.out.println("cPct:         "+cdf.getCompressionPct());
            System.out.println("Cache Size:   "+cdf.confirmCDFCacheSize());

            /**********************************************/
            /* Print out the Global Attribute information */
            /**********************************************/
            Attribute a;
            String    attrName = null;
            int       i;
            long      n = cdf.getNumGattrs();
            Vector    ga = cdf.getGlobalAttributes();
         
            System.out.println("\nGlobal Attributes ("+n+" attributes)\n"+
                               "=========================================");
            i = 0;
            for (Enumeration e = ga.elements() ; e.hasMoreElements() ;) {
                a = (Attribute) e.nextElement();
                n = a.getNumEntries(); 
                if (i == 0) {
                   attrName = a.getName();
                   if (n <= 1)
                       System.out.println (attrName+" ("+n+" entry):");
                   else
                       System.out.println (attrName+" ("+n+" entries):");
                }
                else {
                   String currAttrName = a.getName();;
                   if (currAttrName != attrName) {
                       if (n <= 1)
                          System.out.println (currAttrName+" ("+n+" entry):");
                       else
                          System.out.println (currAttrName+" ("+n+" entries):");
                   }
                } 
                i++;
                Vector ent = a.getEntries();
                for (Enumeration e1 =ent.elements() ; e1.hasMoreElements() ;) {
                    Entry entry = (Entry) e1.nextElement();
                    if (entry != null) {
                       if (entry.getID() == 11) 
                           entry.delete();
                       else {
                          long eDataType = entry.getDataType();
                          System.out.print ("\t"+entry.getID()+" ("+
                                         CDFUtils.getStringDataType(eDataType)+
                                         "/"+entry.getNumElements()+
                                         "): \t");
                          CDFUtils.printData (entry.getData()); 
                          System.out.println (" ");
                       }
                    }
                }
                System.out.println (" ");
            }

            /************************************************/
            /* Print out the Variable Attribute information */
            /************************************************/
            attrName = null;
            n = cdf.getNumVattrs();
            Vector  va = cdf.getVariableAttributes();

            System.out.println("\nVariable Attributes ("+n+" attributes)\n"+
                               "=========================================");
            i = 0;
            for (Enumeration e = va.elements() ; e.hasMoreElements() ;) {
                a = (Attribute) e.nextElement();
                if (i == 0) {
                    attrName = a.getName();
                    System.out.println (attrName+":");
                }
                else {
                    String currAttrName = a.getName();;
                    if (currAttrName != attrName)
                        System.out.println (currAttrName+":");
                }
                i++;
                Vector ent = a.getEntries();
                for (Enumeration e1 =ent.elements() ; e1.hasMoreElements() ;) {
                    Entry entry = (Entry) e1.nextElement();
                    if (entry != null) {
                       long eDataType = entry.getDataType();
                       Variable v = cdf.getVariable(entry.getID());
                       System.out.print ("\t"+v.getName()+" ("+
                                        CDFUtils.getStringDataType(eDataType)+
                                        "/"+entry.getNumElements()+
                                        "): ");

                       CDFUtils.printData (entry.getData());
                       System.out.println (" ");
                    }
                }
                System.out.println (" ");
            }

            /**************************************/
            /* Print out the Variable information */ 
            /**************************************/
            String  varName, dataType;
            int     noOfBlanks;
            long    numDims;
            n = cdf.getNumVars();
            Vector  vars = cdf.getVariables();

            System.out.println("\nVariable Information ("+n+" variables)\n"+
                               "=========================================");
            for (Enumeration e = vars.elements() ; e.hasMoreElements() ;) {
                Variable v = (Variable) e.nextElement();

                varName = v.getName();
                noOfBlanks = maxVarNameLength - varName.length();
                for (i=0; i < noOfBlanks; i++)
                    varName = varName + " ";

                long[] dimSizes = v.getDimSizes();
                dataType = CDFUtils.getStringDataType(v.getDataType());
                dataType = dataType + "/" + String.valueOf(v.getNumElements());
                noOfBlanks = 14 - dataType.length();
                for (i=0; i < noOfBlanks; i++)
                    dataType = dataType + " ";

                numDims = v.getNumDims();
                System.out.print (varName+dataType+ numDims+":[");
                for (i=0; i < numDims; i++) {
                    if (i > 0) System.out.print (",");
                    System.out.print (dimSizes[i]); 
                }
                System.out.print ("]\t");
                // if (numDims == 1) System.out.print ("\t");

                System.out.print((v.getRecVariance() ? "T" : "F")+"/");
                long[] dimVariances = v.getDimVariances();
                for (i=0; i < v.getNumDims(); i++)
                    System.out.print(
                        ((dimVariances[i] == CDFConstants.VARY) ? "T" : "F"));
                System.out.println (" ");
            }

            /**********************************************************/
            /* Print out the Variable data (all variables in the CDF) */ 
            /**********************************************************/
            System.out.println("\n\nVariable Data ("+n+" variables)\n"+
                               "=========================================");
            CDFData data = null;
            long   numRecs, maxRec;
            long[] dimIndices   = {0L};
            long[] dimIntervals = {1L};
            long[] dimSizes     = {1L};

            for (Enumeration e = vars.elements() ; e.hasMoreElements() ;) {
                Variable v = (Variable) e.nextElement();

                if (v.getNumDims() > 0) {
                    dimSizes = v.getDimSizes();
                    dimIntervals = new long[dimSizes.length];
                    dimIndices   = new long[dimSizes.length];
                    for (i=0; i < dimSizes.length; i++) {
                        dimIntervals[i] = 1;
                        dimIndices[i]   = 0;
                    }
                }
                maxRec = v.getMaxWrittenRecord();
                numRecs = v.getNumWrittenRecords();
                varName = v.getName();
                System.out.println (varName);
                for (i=0; i < varName.length(); i++)
                     System.out.print ("-");
                System.out.println (" ");

                if (v.getCompressionType() == NO_COMPRESSION)
                    System.out.println ("Compression:      None");
                else
                    System.out.println ("Compression:      "+
                                        v.getCompression()+" ("+
                                        v.getCompressionPct()+"%)");
                System.out.println ("Pad value:        "+
                                    v.getPadValue());
                System.out.println ("Records:          "+
                                    numRecs+"n/"+maxRec+"x");
                System.out.println ("Allocated:        "+
                                    v.getNumAllocatedRecords()+"n/"+
                                    v.getMaxAllocatedRecord()+"x");
                System.out.println ("Blocking Factor:  "+
                                    v.getBlockingFactor());
                System.out.println ("Sparseness:       "+
                                    CDFUtils.getStringSparseRecord(v));
                System.out.println (" ");

                /********************************************************/
                /*  maxRec represents the last record number for this   */
                /*  variable, not the number of records.                */
                /*                                                      */
                /*  NOTE: maxRec starts at 0, so if the value of maxRec */
                /*        is 2, the actual number of records is 3.      */
                /*        If there are no records exists, the value of  */
                /*        maxRec is -1.                                 */
                /********************************************************/

                for (i=0; i <= maxRec; i++) {
                     System.out.println ("Record # "+i+":");
                     data = v.getHyperDataObject(i, 1, 1,
                                                 dimIndices, 
                                                 dimSizes, 
                                                 dimIntervals);
                     data.dumpData();
                     System.out.println(" ");                
                }
            }

            /**************************************************/
            /*  Print out a few individual variable data and  */
            /*  attribute entries.                            */
            /**************************************************/
            if (fileName.equals("test")) {
               System.out.println("\n\nIndividual Variable Data\n"+
                                   "=================================");

               Variable longitude = cdf.getVariable("Longitude"),   /* T/T */
                        latitude  = cdf.getVariable("Latitude"),    /* F/T */
                        latitude1 = cdf.getVariable("Latitude1"),   /* T/F */
                        time      = cdf.getVariable("Time"),        /* T/T */
                        foo       = cdf.getVariable("foo"),         /* F/F */
                        v         = cdf.getVariable("longitude_dup");

                System.out.println ("longDup.dumpData():");
                System.out.println ("------------------");
                if (v.getNumDims() > 0) {
                    dimSizes     = v.getDimSizes();
                    dimIntervals = new long[dimSizes.length];
                    dimIndices   = new long[dimSizes.length];
                    for (i=0; i < dimSizes.length; i++) {
                        dimIntervals[i] = 1;
                        dimIndices[i]   = 0;
                    }
                }
                maxRec = v.getMaxWrittenRecord();
                for (i=0; i <= maxRec; i++) {
                     System.out.println ("Record # "+i+":");
                     data = v.getHyperDataObject(i, 1, 1,
                                                 dimIndices,
                                                 dimSizes,
                                                 dimIntervals);
                     data.dumpData();
                     System.out.println(" ");
                }
 
               
               System.out.print ("Record #0 for latitude: ");
               CDFUtils.printData (latitude.getRecord (0));
               System.out.println ("");

               System.out.print ("Record #1 for longitude: ");
               CDFUtils.printData (longitude.getRecord (1));
               System.out.println ("");

               System.out.print ("Record #2 for latitude1: ");
               CDFUtils.printData (latitude1.getRecord (2));
               System.out.println ("");

               System.out.print ("Record #0 for foo: ");
               CDFUtils.printData (foo.getRecord (0));
               System.out.println ("\n");

               System.out.print ("1st element of record #0 for latitude: ");
               CDFUtils.printData (latitude.getSingleData(0, new long [] {0}));
               System.out.println ("");

               System.out.print ("2nd element of record #1 for longitude: ");
               CDFUtils.printData (longitude.getSingleData(1, new long [] {1}));
               System.out.println ("");

               System.out.print ("3rd element of record #2 for longitude: ");
               CDFUtils.printData (latitude1.getSingleData(2, new long [] {2}));
               System.out.println ("");

               System.out.print ("1st element of record #0 for foo: ");
               CDFUtils.printData (foo.getSingleData (0, new long [] {0}));
               System.out.println ("\n");

               CDFData dataRecord;
               System.out.print ("(1,0)th element of record #0 for Time: ");
               dataRecord = time.getSingleDataObject(0, new long[] {1,0});
               dataRecord.dumpData();

               System.out.print ("(1,1)th element of record #0 for Time: ");
               Long tValue = (Long) time.getSingleData(0, new long[] {1,1}); 
               System.out.println (tValue);

               System.out.println ("Record #0 for Time: ");
               dataRecord = time.getRecordObject(0L); 
               dataRecord.dumpData();
               System.out.println ("\n");

               System.out.println ("Record #0 for Time: ");
               long[][]  yy = (long [][]) time.getRecord(0L); 
               for (int x=0;x<3; x++)
                 for (int j=0;j<2;j++)
                      System.out.println("["+x+","+j+"] ="+yy[x][j]);

               System.out.println ("\n");

               Variable var;
               var = cdf.getVariable("Temperature2");
               System.out.print ("getScalarData for Temperature2: ");
               CDFUtils.printData (var.getScalarData());
               System.out.println ("\n");

               System.out.print ("getScalarDataObject for Temperature2: ");
               dataRecord = var.getScalarDataObject();
               dataRecord.dumpData();

               var = cdf.getVariable("Temperature");
               System.out.print ("Record #0 for Temperature: ");
               CDFUtils.printData (var.getScalarData(0L));
               System.out.println ("\n");

               System.out.print ("Record #1 for Temperature: ");
               dataRecord = var.getScalarDataObject(1L);
               dataRecord.dumpData();

               var = cdf.getVariable("Delta");
               System.out.println ("HyperGet for Delta: ");
               int[][][] xx = (int [][][]) var.getHyperData (1L, 3L, 1L, 
                                                             new long[] {1, 0},
                                                             new long[] {2, 2},
                                                             new long[] {1, 1});
               for (int x=0;x<2; x++) 
                 for (int j=0;j<2;j++) 
                   for (int k=0;k<2;k++) 
                       System.out.println("["+x+","+j+","+k+"] ="+xx[x][j][k]);

               int[][] zz = (int [][]) var.getHyperData (0L, 3L, 1L,
                                                         new long[] {0, 1},
                                                         new long[] {3, 1},
                                                         new long[] {1, 1});
               System.out.println(" ");
               for (int x=0;x<3; x++)
                 for (int j=0;j<3;j++)
                       System.out.println("["+x+","+j+"] ="+zz[x][j]);


               System.out.println ("\n");
                                 

               System.out.println("\n\nVariable/Global Attribute Entries"+
                                  "\n====================================");

               Attribute  test = cdf.getAttribute("Test"),         /* global */
                          validMin = cdf.getAttribute("validmin"); /* var */

               Entry  tEntry = test.getEntry(15),
                      vEntry = validMin.getEntry(longitude); 

               long attrNum = test.getID();
               Attribute test1 = cdf.getAttribute(attrNum);

               System.out.print (test1.getName()+": \n\t");
               CDFUtils.printData (tEntry.getData());
               System.out.println ("");

               System.out.print ("\nVALIDMIN: \n\tLongitude: ");
               CDFUtils.printData (vEntry.getData());
               System.out.println ("");

               System.out.print ("\nVALIDMAX: \n\tLatitude: ");
               var = cdf.getVariable("Latitude");
               CDFUtils.printData (var.getEntryData("VALIDMAX"));
               System.out.println ("");
            }

            cdf.close();

        } catch (Exception e) {
             System.out.println (e);
        }
    }
}
